int button = 3;

int currentButtonState = 0,
    previousButtonState = 0;

long int  pressTime = 0,
          releaseTime = 0;

//For semantic labels
#define SHORT 0
#define MEDIUM 1
#define LONG 2

//Need my human experience to know what is reasonable here:
const int shortMaximum = 200,
          mediumMaximum = 600,
          longMinimum = mediumMaximum; //doesn't "long" mean "longer than medium"?

int pressHistoryIndex = 0;
int releaseHistoryIndex = 0;
const int historySize = 3;
int recentPresses[historySize] = {LONG, LONG, LONG};
int recentReleases[historySize] = {LONG, LONG, LONG};

void setup() {
  // put your setup code here, to run once:
  pinMode(button, INPUT_PULLUP);
  Serial.begin(9600);
  releaseTime = millis();
}

void loop() {
  // put your main code here, to run repeatedly:
  currentButtonState = digitalRead(button);
  if (currentButtonState != previousButtonState) {
    if (currentButtonState != HIGH) {
      pressTime = millis(); //record time of press
      long int duration = millis() - releaseTime;
      if (duration > 1) {
        Serial.print("Change: ");
        Serial.print("Pressed, ");
        Serial.print(duration);
        Serial.print(" ms, ");
        printDurationClassification(duration);
        Serial.println();
        recordPress(duration);
      }
    } else {
      releaseTime = millis(); //record time of release
      long int duration = millis() - pressTime;
      if (duration > 1) { //debounce. How do I *know* 1ms is OK debouce time?
        Serial.print("Change: ");
        Serial.print("Released, ");
        Serial.print(duration);
        Serial.print(" ms, ");
        printDurationClassification(duration);
        Serial.println();
        displayHistoryClassification(weighHistory());
        Serial.println();
        recordRelease(duration);
      }
    }
    previousButtonState = currentButtonState;
  }
  //delay(2);
}

int classifyDuration(long int duration) {
  if (duration < shortMaximum) { //How do I *know* <200ms "is" "short"?
    return SHORT;
  } else if (duration >= shortMaximum && duration <= mediumMaximum) {
    return MEDIUM;
  } else if (duration > longMinimum) {
    return LONG;
  }
  return -1; //shouldn't happen
}

void printDurationClassification(int d) {
  switch (classifyDuration(d)) {
    case SHORT:
      Serial.print("SHORT");
      break;
    case MEDIUM:
      Serial.print("MEDIUM");
      break;
    case LONG:
      Serial.print("LONG");
      break;
  }
}

int weighHistory() {
  int total = 0;
  for (int i = 0; i<3; i++) {
    total+= recentPresses[i] + recentReleases[i];
  }
  return total;
}

int displayHistoryClassification(int d) {
  Serial.print("Raw weight: ");
  Serial.print(d);
  Serial.print(", Classification: ");
  if (d == 12) { //all press and release times are "long"
    Serial.println("Languid");
  } else if (d == 0) { //all press and release times are "short"
    Serial.println("Frantic");
  } else {
    Serial.println("Transitional"); //distinguish more states here:
  }
}

void recordPress(int d) {
  recentPresses[pressHistoryIndex] = classifyDuration(d);
  pressHistoryIndex = (pressHistoryIndex + 1) % historySize;
}

void recordRelease(int d) {
  recentReleases[releaseHistoryIndex] = classifyDuration(d);
  releaseHistoryIndex = (releaseHistoryIndex + 1) % historySize;
}
